@@ -1,6 +1,7 @@
 # ActivitySim
 # See full license in LICENSE.txt.
 import logging
+import os
 
 import numpy as np
 import pandas as pd
@@ -15,8 +16,8 @@ logger = logging.getLogger(__name__)
 @inject.injectable(cache=True)
 def tdd_alts():
     # right now this file just contains the start and end hour
-    f = config.config_file_path('tour_departure_and_duration_alternatives.csv')
-    df = pd.read_csv(f)
+    file_path = config.config_file_path('tour_departure_and_duration_alternatives.csv')
+    df = pd.read_csv(file_path)
 
     df['duration'] = df.end - df.start
 
@@ -26,6 +27,32 @@ def tdd_alts():
     return df
 
 
+@inject.injectable(cache=True)
+def tdd_alt_segments():
+
+    # tour_purpose,time_period,start,end
+    # work,EA,3,5
+    # work,AM,6,8
+    # ...
+    # school,PM,15,17
+    # school,EV,18,22
+
+    file_path = config.config_file_path('tour_departure_and_duration_segments.csv', mandatory=False)
+
+    if file_path:
+
+        df = pd.read_csv(file_path, comment='#')
+
+        # - NARROW
+        df['start'] = df['start'].astype(np.int8)
+        df['end'] = df['end'].astype(np.int8)
+
+    else:
+        df = None
+
+    return df
+
+
 @inject.table()
 def person_windows(persons, tdd_alts):
 
